import {Dispatch, SetStateAction, useEffect, useState} from "react";
import Card from "./Card";
import {
    correctSelection,
    incorrectSelection,
    incorrectSymptomsCards,
    nextButton,
    symptomsCards,
    symptomsCardTitle
} from "../../public/lit/texts";
import {useRecoilValue} from "recoil";
import {languageAtom} from "../atoms/languageAtom";
import {Texts} from "../hooks/useAnimate";
import {useTranslate} from "../hooks/useTranslate";

interface SymptomsTestProps {
    handleNext: () => void,
    setScore: Dispatch<SetStateAction<number>>,
    score: number
}

interface Card {
    id: string;
    title: {
        EN: string;
        SK: string;
    };
    image: string;
}

const SymptomsCardTest = ({handleNext, setScore, score}: SymptomsTestProps) => {
    const language = useRecoilValue(languageAtom);
    const [selectedSymptoms, setSelectedSymptoms] = useState<string[]>([]);
    const nextButtonText = useTranslate(nextButton);
    const title = useTranslate(symptomsCardTitle);
    const correctSelectionText = useTranslate(correctSelection);
    const incorrectSelectionText = useTranslate(incorrectSelection);
    const [cards, setCards] = useState<Card[]>([]);

    useEffect(() => {
        const shuffleArray = (array: Card[]): Card[] => {
            for (let i = array.length - 1; i > 0; i--) {
                const j = Math.floor(Math.random() * (i + 1));
                [array[i], array[j]] = [array[j], array[i]];
            }
            return array;
        };
        setCards(shuffleArray([...symptomsCards, ...incorrectSymptomsCards]));
    }, []);

    const handleCardClick = (symptomId: string) => {
        if (selectedSymptoms.includes(symptomId)) {
            setSelectedSymptoms((prev) => prev.filter((id) => id !== symptomId));
        } else {
            setSelectedSymptoms((prev) => [...prev, symptomId]);
        }
    };

    const isCorrect = () => {
        return (
            selectedSymptoms.length === symptomsCards.length &&
            symptomsCards.every((symptom) =>
                selectedSymptoms.includes(symptom.id)
            ))
            ;
    };

    return (
        <div>
            <p className="text-lg align-center block">{title}</p>
            <div className="grid grid-cols-6 gap-4 my-4 mx-40">
                {cards.map(({id, title, image}) => (
                    <Card
                        key={id}
                        image={image}
                        text={title[language as keyof Texts]}
                        alt={id}
                        isSelected={selectedSymptoms.includes(id)}
                        onClick={() => handleCardClick(id)}
                    />
                ))}
            </div>
            {selectedSymptoms.length > 0 && (
                <p className={`my-4 ${isCorrect() ? 'text-green-500' : 'text-red-500'}`}>
                    {isCorrect()
                        ? `${correctSelectionText}`
                        : `${incorrectSelectionText}`}
                </p>
            )}
            {isCorrect() && (
                <button className="border-4 border-black font-primary bg-yellow-main"
                    onClick={() => {
                        setScore(score + 1);
                        handleNext()
                    }}>
                    {nextButtonText}
                </button>
            )}
        </div>
    );
}
export default SymptomsCardTest;
